# Demo 3: Predictive Maintenance mit R - Teil 2

# 0 Benötigte Pakete importieren ----
# Lade benötigte Pakete
library(rpart)
library(caret)

# zur Bestimmung der AUC 
# der ROC-Kurve 
library(ROCR)  

# Lade externe Funktion
source("CreateSamples.r")

# 1 Datenvorbereitung ----
# Lese CSV-Datei und speichere 
# sie unter dem Namen "demo3"
demo3=read.csv2("automotive_data_140.csv",
                header = TRUE,sep = ";",dec = ".")

# Wähle Merkmale aus
# Erstelle Trainingsdaten demo3training
demo3training=demo3[ ,c(-1,-25)] 

# Überprüfe die Struktur von demo3training
str(demo3training)

# 2 Kreuzvalidierung(CV) ----
# 2.1 Vorbereitung ---- 

# Eingabe-Variable:
# Anzahl der Teildatensätze in CV
K=10

# Erzeuge Teildatensätze mittels 
# externer Funktion
folds=createCVFolds(demo3training, 
                    K,seed=2012)

# Ausgabematrix der 
# Performance-Kennzahlen
m_perf=matrix(nrow = K, ncol = 4)

# Spaltennamen festlegen
colnames(m_perf)=c("recall", "precision"
                   ,"accuracy", "auc")

# 2.2 Durchführung ----

for (i in 1:K){
  # Erstelle Entscheidungsbaum 
  # mit dem CART-Algorithmus
  # und k-1 Trainingsdatensätzen
  fit = rpart(Ausfall ~ ., 
              data=demo3training[folds!=i,], 
              method='class', 
              parms=list(split="information"),
              control=list(minsplit = 15))
  
  # Vorhersage für den 
  # ausgelassenen Teildatensatz/Testdaten
  testdata=demo3training[folds==i,]
  pred=predict(fit, newdata=testdata, 
               type="class")
  
  # Erzeuge Konfusionsmatrix 
  # mit caret-Paket
  m_conf=confusionMatrix(
           pred,as.factor(testdata$Ausfall),
           positive="ja")
  
  # Speichere die Kennzahlen 
  # des i-ten Durchlaufs der CV
  m_perf[i,1]=m_conf$byClass[6] #recall
  m_perf[i,2]=m_conf$byClass[5] #precision
  m_perf[i,3]=m_conf$overall[1] #accuracy
  
  # AUC, ROC: Vorbereitungen
  # Wahrscheinlichkeiten für "Ausfall" 
  pred.prob=predict(fit, newdata=testdata, 
                    type="prob")
  
  # Vektor mit TRUE/FALSE für Ausfall 
  # in den Beobachtungen der Testdaten
  obs=(testdata$Ausfall=="ja")
  
  # Verwende ROCR-Paket zur Berechnung der AUC
  prediction.obj=prediction(pred.prob[,1],obs)
  perf.auc=performance(prediction.obj, 
                       measure = "auc")
  
  # Speichere die AUC des i-ten 
  # Durchlaufs der CV
  m_perf[i,4]=as.double(perf.auc@y.values)
}

# 3 Performance-Kennzahlen ----
# Mittelung der Performance-Kennzahlen 
# über alle CV-Durchläufe
perf=colSums(m_perf)/K

# Ausgabe der Performance-Kennzahlen
perf




